import copy

import requests
from dateutil import tz

from .classes import AlchemerObject, Survey, ContactList


class AlchemerSession(requests.Session):
    def __init__(
        self,
        api_version,
        api_token,
        api_token_secret,
        auth_method="api_key",
        time_zone=None,
    ):
        self.api_version = api_version
        self.base_url = f"https://api.alchemer.com/{api_version}"
        self.time_zone = tz.gettz(time_zone)

        if api_version != "v5":
            raise NotImplementedError(
                "This library currently only works with v5+"
            )  # TODO: add < v5

        if auth_method == "api_key":
            self.auth_params = {
                "api_token": api_token,
                "api_token_secret": api_token_secret,
            }
        elif auth_method == "oauth":
            raise NotImplementedError(
                "This library currently only works with 'api_key' authentication"
            )  # TODO: add oauth

        super(AlchemerSession, self).__init__()

    def request(self, method, url, params, *args, **kwargs):
        params.update(self.auth_params)
        return super(AlchemerSession, self).request(
            method=method, url=url, params=params, *args, **kwargs
        )

    def _api_call(self, method, url, params):
        try:
            r = self.request(method, url=url, params=params)
            r.raise_for_status()
            return r.json()
        except Exception as xc:
            raise xc

    def _api_get(self, url, params):
        params = copy.deepcopy(params)
        return self._api_call(method="GET", url=url, params=params)

    def _api_list(self, url, params):
        original_params = params
        req_params = copy.deepcopy(params)        
        all_data = []
        while True:
            response = self._api_get(url=url, params=req_params)

            total_pages = response.get("total_pages", 1)
            page = response.get("page", 1)
            data = response.get("data")
            if isinstance(data, dict):
                data = [data]
            elif data is None:
                data = []

            all_data.extend(data)

            if page == total_pages:
                break
            elif total_pages == 0:
                break
            elif "page" in original_params:
                break
            else:
                req_params.update({"page": page + 1})

        return all_data

    @property
    def account(self):
        return AlchemerObject(session=self, name="account")

    @property
    def account_teams(self):
        return AlchemerObject(session=self, name="accountteams")

    @property
    def account_user(self):
        return AlchemerObject(session=self, name="accountuser")

    @property
    def domain(self):
        return AlchemerObject(session=self, name="domain")

    @property
    def sso(self):
        return AlchemerObject(session=self, name="sso")

    @property
    def survey(self):
        return Survey(session=self, name="survey")

    @property
    def survey_theme(self):
        return AlchemerObject(session=self, name="surveytheme")

    @property
    def contact_list(self):
        return ContactList(session=self, name="contactlist")

    @property
    def contact_custom_field(self):
        return AlchemerObject(session=self, name="contactcustomfield")



import copy

from dateutil import parser, tz

TZINFOS = {
    "EST": tz.gettz("US/Eastern"),
    "EDT": tz.gettz("US/Eastern"),
    "CST": tz.gettz("US/Central"),
    "CDT": tz.gettz("US/Central"),
    "MST": tz.gettz("US/Mountain"),
    "MDT": tz.gettz("US/Mountain"),
    "PST": tz.gettz("US/Pacific"),
    "PDT": tz.gettz("US/Pacific"),
}


class AlchemerObject(object):
    def __init__(self, session, name, **kwargs):
        self.__name__ = name
        self.__parent = kwargs.pop("parent", None)
        self._session = getattr(self.__parent, "_session", session)
        self._data = {}
        self._filters = []
        self.url = f"{getattr(self.__parent, 'url', session.base_url)}/{name}"

    def _prepare_filters(self, params):
        for f in self._filters:
            params.update(f)
        return params

    def get(self, id, **kwargs):
        params = copy.deepcopy(kwargs.get("params", {}))

        self.url = f"{self.url}/{id}"
        self.data = self._session._api_get(url=self.url, params=params).get("data")

        for k, v in self.data.items():
            try:
                v = parser.parse(v, tzinfos=TZINFOS)
                if not v.tzinfo and self._session.time_zone:
                    v = v.replace(tzinfo=self._session.time_zone)
            except:
                pass
            setattr(self, k, v)

        return self

    def list(self, **kwargs):
        params = copy.deepcopy(kwargs.get("params", {}))
        params = self._prepare_filters(params)
        return self._session._api_list(
            url=self.url,
            params=params,
        )

    def create(self, **kwargs):
        params = copy.deepcopy(kwargs.get("params", {}))
        return self._session._api_call(method="PUT", url=self.url, params=params)

    def update(self, id, **kwargs):
        params = copy.deepcopy(kwargs.get("params", {}))
        self.url = f"{self.url}/{id}"
        return self._session._api_call(method="POST", url=self.url, params=params)

    def delete(self, id):
        self.url = f"{self.url}/{id}"
        return self._session._api_call(method="DELETE", url=self.url, params={})

    def filter(self, field, operator, value):
        i = len(self._filters)
        self._filters.append(
            {
                f"filter[field][{i}]": field,
                f"filter[operator][{i}]": operator,
                f"filter[value][{i}]": value,
            }
        )
        return self


class Survey(AlchemerObject):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @property
    def campaign(self):
        return SurveyCampaign(parent=self, session=self._session, name="surveycampaign")

    @property
    def page(self):
        return AlchemerObject(parent=self, session=self._session, name="surveypage")

    @property
    def question(self):
        return SurveyQuestion(parent=self, session=self._session, name="surveyquestion")

    @property
    def quota(self):
        return AlchemerObject(parent=self, session=self._session, name="quotas")

    @property
    def report(self):
        return AlchemerObject(parent=self, session=self._session, name="surveyreport")

    @property
    def reporting(self):
        return AlchemerObject(parent=self, session=self._session, name="reporting")

    @property
    def response(self):
        return AlchemerObject(parent=self, session=self._session, name="surveyresponse")

    @property
    def results(self):
        return AlchemerObject(parent=self, session=self._session, name="results")

    @property
    def statistic(self):
        return AlchemerObject(
            parent=self, session=self._session, name="surveystatistic"
        )


class SurveyQuestion(AlchemerObject):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @property
    def option(self):
        return AlchemerObject(parent=self, session=self._session, name="surveyoption")


class SurveyCampaign(AlchemerObject):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @property
    def contact(self):
        return AlchemerObject(
            parent=self, session=self._session, name="surveycontact"
        )  # TODO: returns None?

    @property
    def email_message(self):
        return AlchemerObject(parent=self, session=self._session, name="emailmessage")


class ContactList(AlchemerObject):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @property
    def contact(self):
        return AlchemerObject(
            parent=self, session=self._session, name="contactlistcontact"
        )


class Reporting(AlchemerObject):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @property
    def element(self):
        return AlchemerObject(parent=self, session=self._session, name="reportelement")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['alchemer']

package_data = \
{'': ['*']}

install_requires = \
['python-dateutil>=2.8.2,<3.0.0', 'requests>=2.26.0,<3.0.0']

setup_kwargs = {
    'name': 'alchemer',
    'version': '0.5.4',
    'description': '',
    'long_description': None,
    'author': 'Charlie Bini',
    'author_email': '5003326+cbini@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
